unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

type
  TForm1 = class(TForm)
    rgComPort: TRadioGroup;
    StatusBar1: TStatusBar;
    FlashMemo: TMemo;
    ProgressBar1: TProgressBar;
    GroupBox1: TGroupBox;
    cbInvMosi: TCheckBox;
    cbInvMiso: TCheckBox;
    cbInvSck: TCheckBox;
    GroupBox2: TGroupBox;
    cbSleep: TCheckBox;
    tbDelay: TTrackBar;
    btnOpenFile: TButton;
    eProgFile: TEdit;
    btnOpenDialog: TButton;
    OpenDialog1: TOpenDialog;
    GroupBox3: TGroupBox;
    btnReadSig: TButton;
    btnReadEE: TButton;
    btnReadFlash: TButton;
    GroupBox4: TGroupBox;
    btnWriteEE: TButton;
    btnErase: TButton;
    btnWriteFlash: TButton;
    btnProgram: TButton;
    btnVerify: TButton;
    EEMemo: TMemo;
    cbLoadEEP: TCheckBox;
    btnDispBuf: TButton;
    btnProgLock: TButton;
    Label1: TLabel;
    cbDevice: TComboBox;
    btnAbout: TButton;
    GroupBox5: TGroupBox;
    btnReadFuses: TButton;
    btnWriteFuses: TButton;
    eFuses: TEdit;
    btnAVRreset: TButton;
    btnTestBuf: TButton;
    btnEditEE: TButton;
    btnBootloader: TButton;
    GroupBox6: TGroupBox;
    btnReadLock: TButton;
    btnWriteLock: TButton;
    eLock: TEdit;
    GroupBox7: TGroupBox;
    btnReadExtFuses: TButton;
    btnWriteExtFuses: TButton;
    eExtFuses: TEdit;
    procedure btnReadSigClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure rgComPortClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnReadEEClick(Sender: TObject);
    procedure btnReadFlashClick(Sender: TObject);
    procedure btnWriteEEClick(Sender: TObject);
    procedure btnEraseClick(Sender: TObject);
    procedure btnWriteFlashClick(Sender: TObject);
    procedure btnOpenDialogClick(Sender: TObject);
    procedure btnOpenFileClick(Sender: TObject);
    procedure btnProgramClick(Sender: TObject);
    procedure btnVerifyClick(Sender: TObject);
    procedure btnDispBufClick(Sender: TObject);
    procedure btnProgLockClick(Sender: TObject);
    procedure cbDeviceChange(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure btnReadFusesClick(Sender: TObject);
    procedure btnWriteFusesClick(Sender: TObject);
    procedure cbInvMosiClick(Sender: TObject);
    procedure btnAVRresetClick(Sender: TObject);
    procedure btnTestBufClick(Sender: TObject);
    procedure tbDelayChange(Sender: TObject);
    procedure btnEditEEClick(Sender: TObject);
    procedure btnBootloaderClick(Sender: TObject);
    procedure btnReadLockClick(Sender: TObject);
    procedure btnWriteLockClick(Sender: TObject);
    procedure btnReadExtFusesClick(Sender: TObject);
    procedure btnWriteExtFusesClick(Sender: TObject);
  private
    flashbuf: array[0..$ffff] of word;    // 32kb
    fbufsize: integer;
    eebuf: array[0..$fff] of byte;        // 1kb
    ebufsize: integer;
    devflashsize: integer;
    deveesize: integer;
    fpagesize: integer;

    procedure AppException(Sender: TObject; E: Exception);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses MotSRec, Registry, Unit2;

// -----------------------------------------------------------------------------
// --- GUI handlers ------------------------------------------------------------
// -----------------------------------------------------------------------------

procedure TForm1.AppException(Sender: TObject; E: Exception);
begin
  StatusBar1.SimpleText := E.Message;
end;

procedure TForm1.FormCreate(Sender: TObject);
var r: TRegistry;
    i, cp: integer;
begin
  devflashsize := 512;
  deveesize := 64;
  fpagesize := 0;
  cp := 0;
  
  cbDevice.Items.Clear;
  for i := 0 to devnum-1 do cbDevice.Items.Add(devnames[i]);
  cbDevice.Items.Add('Custom');
  cbDevice.ItemIndex := 0;
  Application.OnException := AppException;

  r := TRegistry.Create;
  try
    if r.OpenKey('\Software\Serial AVR programmer', false) then
    begin
      eProgFile.Text := r.ReadString('LastFile');
      cbLoadEEP.Checked := r.ReadBool('LoadEEPROM');
      delaycnt := r.ReadInteger('Timing');
      cbDevice.ItemIndex := r.ReadInteger('AVRdevice');
      cp := r.ReadInteger('ComPort');
    end;
  finally
    r.Free;
  end;

  cbInvMiso.Checked := invmiso;
  cbInvMosi.Checked := invmosi;
  cbInvSck.Checked := invsck;
  tbDelay.Position := delaycnt;

  cbDevice.OnChange(nil);
  if (rgComPort.ItemIndex = cp) then rgComPort.OnClick(rgComPort)
                                else rgComPort.ItemIndex := cp;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
var r: TRegistry;
begin
  r := TRegistry.Create;
  try
    r.OpenKey('\Software\Serial AVR programmer', true);
    r.WriteString('LastFile', eProgFile.Text);
    r.WriteBool('LoadEEPROM', cbLoadEEP.Checked);
    r.WriteInteger('ComPort', rgComPort.ItemIndex);
    r.WriteInteger('Timing', tbDelay.Position);
    r.WriteInteger('AVRdevice', cbDevice.ItemIndex);
  finally
    r.Free;
  end;
end;

procedure TForm1.rgComPortClick(Sender: TObject);
var c: char;
begin
  CloseCom();
  CloseLpt();

  if (rgComPort.ItemIndex = 0) then
  begin
    OpenLpt();

    StatusBar1.SimpleText := 'Make sure giveio.sys is running...';
  end else
  begin
    c := Chr(48 + rgComPort.ItemIndex);

    OpenCom(c);

    StatusBar1.SimpleText := 'Reset the chip and press Read signature to enter programming mode';
  end;
end;

procedure TForm1.btnReadSigClick(Sender: TObject);
var b: byte;
    i,j: integer;
    s: string;
begin
  AVRprogEnable();

  s := '';
  j := 0;

  for i := 0 to 2 do
  begin
    b := AVRreadSignature(i);
    j := j * $100 + b;

    s := s + inttohex(b, 2) + ' ';
  end;

  StatusBar1.SimpleText := 'Signature bytes: ' + s + '     (' + AVRdeviceName(j)+ ')';
end;

procedure TForm1.btnReadEEClick(Sender: TObject);
var b: byte;
    i: integer;
    s: string;
begin
  StatusBar1.SimpleText := 'Reading EEPROM ...';

  ProgressBar1.Position := 0;
  ProgressBar1.Max := deveesize;
  s := '';

  for i := 0 to deveesize-1 do
  begin
    b := AVRreadEE(i);
    s := s + inttohex(b, 2) + ' ';

    ProgressBar1.StepIt;
  end;

  EEMemo.Text := s;
  ProgressBar1.Position := 0;

  StatusBar1.SimpleText := 'Ready';
end;

procedure TForm1.btnReadFlashClick(Sender: TObject);
var w: word;
    i: integer;
    s: string;
begin
  StatusBar1.SimpleText := 'Reading Flash ...';

  ProgressBar1.Position := 0;
  ProgressBar1.Max := devflashsize;
  s := '';

  for i := 0 to devflashsize-1 do
  begin
    w := AVRreadProg(i);
    s := s + inttohex(w, 4) + ' ';

    ProgressBar1.StepIt;
  end;

  FlashMemo.Text := s;
  ProgressBar1.Position := 0;

  StatusBar1.SimpleText := 'Ready';
end;

procedure TForm1.btnWriteEEClick(Sender: TObject);
var b: byte;
    i: integer;
    s: string;
begin
  StatusBar1.SimpleText := 'Programming EEPROM ...';

  ProgressBar1.Position := 0;
  ProgressBar1.Max := deveesize;
  s := '';

  for i := 0 to deveesize-1 do
  begin
    b := eebuf[i];
    AVRwriteEE(i,b);
    s := s + inttohex(b, 2) + ' ';

    ProgressBar1.StepIt;
  end;

  ProgressBar1.Position := 0;

  StatusBar1.SimpleText := 'Ready';
end;

procedure TForm1.btnEraseClick(Sender: TObject);
begin
  StatusBar1.SimpleText := 'Erasing chip ...';

  AVRchipErase();

  StatusBar1.SimpleText := 'Ready';
end;

procedure TForm1.btnWriteFlashClick(Sender: TObject);
var i: integer;
    x: array[0..127] of word;
begin
  StatusBar1.SimpleText := 'Programming Flash ...';

  ProgressBar1.Position := 0;
  ProgressBar1.Max := devflashsize;

  if (fpagesize = 0) then
  begin
    for i := 0 to devflashsize-1 do
    begin
      AVRwriteProg(i, flashbuf[i]);

      ProgressBar1.StepIt;
    end;
  end else
  begin
    for i := 0 to (devflashsize div fpagesize)-1 do
    begin
      Move(flashbuf[i*fpagesize], x, fpagesize*2);
      AVRwritePage(x, i, fpagesize);

      ProgressBar1.StepBy(fpagesize);
    end;
  end;

  ProgressBar1.Position := 0;

  StatusBar1.SimpleText := 'Ready';
end;

procedure TForm1.btnOpenDialogClick(Sender: TObject);
begin
  if OpenDialog1.Execute then
  begin
    eProgFile.Text := OpenDialog1.FileName;
    btnOpenFile.Click;
  end;
end;

procedure TForm1.btnOpenFileClick(Sender: TObject);
var i: integer;
    temp: array[0..$1ffff] of byte;
    s: string;
begin
  i := ReadSRec(eProgFile.Text, temp);
  if (i mod 2) > 0 then raise Exception.Create('Invalid odd length file!');
  Move(temp, flashbuf, sizeof(flashbuf));
  fbufsize := i div 2;

  StatusBar1.SimpleText := 'Loaded ' + inttostr(fbufsize) + ' words from ' + eProgFile.Text;

  if cbLoadEEP.Checked then
  begin
    s := Copy(eProgFile.Text, 1, Length(eProgFile.Text)-3) + 'eep';
    i := ReadSRec(s, eebuf);
    ebufsize := i;

    StatusBar1.SimpleText := StatusBar1.SimpleText + ' and ' + inttostr(ebufsize) + ' bytes from ' + s;
  end else
  begin
    FillChar(eebuf, sizeof(eebuf), $ff);
    ebufsize := 0;
  end;

  btnDispBuf.Click;
end;

procedure TForm1.btnProgramClick(Sender: TObject);
begin
  btnAVRreset.Click;
  btnReadSig.Click;
  btnErase.Click;
  btnAVRreset.Click;
  btnReadSig.Click;
  btnWriteFlash.Click;
  btnWriteEE.Click;
  btnVerify.Click;
end;

procedure TForm1.btnVerifyClick(Sender: TObject);
var i: integer;
    w: word;
    b: byte;
begin
  ProgressBar1.Position := 0;
  ProgressBar1.Max := fbufsize;
  for i := 0 to fbufsize-1 do
  begin
    w := AVRreadProg(i);

    if w <> flashbuf[i] then raise Exception.Create('Flash mismatch at 0x' + IntToHex(i, 4) + ' buffer = 0x' + IntToHex(flashbuf[i],4) + ' flash = 0x' + IntToHex(w, 4));

    ProgressBar1.StepIt;
  end;

  if (ebufsize > 0) then
  begin
    ProgressBar1.Position := 0;
    ProgressBar1.Max := ebufsize;
    for i := 0 to ebufsize-1 do
    begin
      b := AVRreadEE(i);

      if b <> eebuf[i] then raise Exception.Create('EEPROM mismatch at 0x' + IntToHex(i, 4));

      ProgressBar1.StepIt;
    end;

    ProgressBar1.Position := 0;
  end;

  StatusBar1.SimpleText := 'Verify OK';
end;

procedure TForm1.btnDispBufClick(Sender: TObject);
var s: string;
    i,j: integer;
begin
  s := '';
//  j := devflashsize-1;
//  if (j > 4*1024-1) then j := 4*1024-1;

  i := 0;
  while flashbuf[i] = $ffff do i := i+1;
  j := i + 4*1024;
  if j > devflashsize-1 then j := devflashsize-1;

  FlashMemo.Clear;
  FlashMemo.Lines.Add('Start: ' + inttohex(i,4));
  while i <= j do
  begin
    s := s + inttohex(flashbuf[i],4) + ' ';
    i := i + 1;
  end;
  FlashMemo.Lines.Add(s);
  FlashMemo.Lines.Add('End: ' + inttohex(i-1,4));

  s := '';
  for i := 0 to deveesize-1 do s := s + inttohex(eebuf[i],2) + ' ';
  EEMemo.Text := s;
end;

procedure TForm1.btnTestBufClick(Sender: TObject);
var i: integer;
begin
  for i := 0 to devflashsize-1 do
    flashbuf[i] := i;

  for i := 0 to deveesize-1 do
    eebuf[i] := i;

  btnDispBuf.Click();
end;

procedure TForm1.btnProgLockClick(Sender: TObject);
begin
  AVRlock();
end;

procedure TForm1.cbDeviceChange(Sender: TObject);
var nfs,nees: integer;
    s: string;
begin
  if cbDevice.ItemIndex < devnum then
  begin
    devflashsize := devflssize[cbDevice.ItemIndex];
    deveesize    := deveepsize[cbDevice.ItemIndex];
    fpagesize := devfpagesize[cbDevice.ItemIndex];
  end else
  begin
    s := IntToStr(devflashsize);
    if InputQuery('Custom device','Enter device flash size in words:', s) then
    begin
      nfs := StrToInt(s);
      s := IntToStr(deveesize);
      if InputQuery('Custom device','Enter device eeprom size in bytes:', s) then
      begin
        nees := StrToInt(s);
        s := IntToStr(fpagesize);

        if InputQuery('Custom device','Enter flash page size (0 = non-paged flash):', s) then
        begin
          fpagesize := StrToInt(s);
          deveesize := nees;
          devflashsize := nfs;
        end;
      end;
    end;
  end;
  StatusBar1.SimpleText := 'Flash size ' + IntToStr(devflashsize)
  + ' words, page size ' + IntToStr(fpagesize)
  + ' words, eeprom size ' + IntToStr(deveesize) + ' bytes';
end;

procedure TForm1.btnAboutClick(Sender: TObject);
begin
  MessageDlg('Serial AVR programmer'+#13+#13+
             'by s56wix@hamradio.si'+#13+#13+
             'Homepage:'+#13+
             'lea.hamradio.si/~s56wix/avrprog/'
             , mtInformation, [mbOK], 0);
end;

procedure TForm1.btnReadFusesClick(Sender: TObject);
begin
  eFuses.Text := IntToHex(AVRreadFuses(), 4);
end;

procedure TForm1.btnWriteFusesClick(Sender: TObject);
begin
  AVRprogFuses(StrToInt('$' + eFuses.Text));
end;

procedure TForm1.cbInvMosiClick(Sender: TObject);
begin
  invmosi := cbInvMosi.Checked;
  invmiso := cbInvMiso.Checked;
  invsck := cbInvSck.Checked;
end;

procedure TForm1.btnAVRresetClick(Sender: TObject);
begin
  if rgComPort.ItemIndex <> 0 then
  begin
    ShowMessage('Reset the chip manually...');
    // raise Exception.Create('Cannot reset serial port based programmer');
  end else
  begin
    OpenLpt();
  end;
end;

procedure TForm1.tbDelayChange(Sender: TObject);
begin
  delaycnt := tbDelay.Position;
end;

procedure TForm1.btnEditEEClick(Sender: TObject);
var eea: word;
    eed: byte;
    s: string;
begin
  s := '0';
  if InputQuery('Change EEPROM contents','Address:', s) then
  begin
    eea := StrToInt(s);
    if (eea >= deveesize) then raise Exception.Create('Invalid EEPROM address');
    s := IntToStr(eebuf[eea]);
    if InputQuery('Change EEPROM contents','Data:', s) then
    begin
      eed := StrToInt(s);
      //if (eed < 0)or(eed > 255) then raise Exception.Create('Invalid data');
      eebuf[eea] := eed;
      btnDispBuf.Click;
    end;
  end;
end;

procedure TForm1.btnBootloaderClick(Sender: TObject);
var i: integer;
    x: array[0..127] of word;
begin
  if (fpagesize = 0) then
    raise Exception.Create('Cannot program this device with bootloader');

  StatusBar1.SimpleText := 'Programming Flash ...';

  ProgressBar1.Position := 0;
  ProgressBar1.Max := devflashsize;

  ShowMessage('Press and hold RESET');
  AVRblInit;
  ShowMessage('Release RESET and press ok');
  AVRblStart;

  for i := 0 to (devflashsize div fpagesize)-1 do
  begin
    Move(flashbuf[i*fpagesize], x, fpagesize*2);
    AVRblWritePage(x, i, fpagesize);

    ProgressBar1.StepBy(fpagesize);
  end;

  ProgressBar1.Position := 0;

  StatusBar1.SimpleText := 'Ready';
end;

procedure TForm1.btnReadLockClick(Sender: TObject);
begin
  eLock.Text := IntToHex(AVRreadLockBits(), 2);
end;

procedure TForm1.btnWriteLockClick(Sender: TObject);
begin
  AVRprogLockBits(StrToInt('$' + eLock.Text));
end;

procedure TForm1.btnReadExtFusesClick(Sender: TObject);
begin
  eExtFuses.Text := IntToHex(AVRreadExtFuses(), 2);
end;

procedure TForm1.btnWriteExtFusesClick(Sender: TObject);
begin
  AVRprogExtFuses(StrToInt('$' + eExtFuses.Text));
end;

end.
