unit Unit2;

interface

uses Windows, SysUtils;

// device definitions

const devnum = 9;
const kb = 1024;
const lptp = $378;

const devnames: array[0..devnum-1] of string =
('AT90S1200','AT90S2313','AT90S4433','AT90S8515','ATmega8515','ATmega8','AT90CAN128','ATtiny26','ATmega128');

// flash size in words!
const devflssize: array[0..devnum-1] of integer =
(512,1*kb,2*kb,4*kb,4*kb,4*kb,64*kb,1*kb,64*kb);

// eeprom size in bytes
const deveepsize: array[0..devnum-1] of integer =
(64,128,256,512,512,512,4*kb,128,4*kb);

// flash page size in words (0 if no paging)
const devfpagesize: array[0..devnum-1] of integer =
(0,0,0,0,32,32,128,16,128);

const devsig: array[0..devnum-1] of integer =
($1e9001,$1e9101,$1e9203,$1e9301,$1e9306,$1e9307,$1e9781,$1e9109,$1e9702);

var
  delaycnt: integer;
  invsck, invmosi, invmiso: boolean;

  procedure OpenLpt();
  procedure CloseLpt();

  procedure OpenCom(c: char);
  procedure CloseCom();

  function AVRdeviceName(a: integer): string;
  procedure AVRprogEnable();
  function AVRreadSignature(a: byte): byte;
  procedure AVRchipErase();
  function AVRreadProgByte(a: integer; high: boolean): byte;
  function AVRreadProg(a: integer): word;
  procedure AVRwriteProg(a: integer; w: word);
  function AVRreadEE(a: integer): byte;
  procedure AVRwriteEE(a: integer; b: byte);
  procedure AVRlock();
  procedure AVRwritePage(d: array of word; pa: word; ps: byte);
  function AVRreadFuses(): word;
  procedure AVRprogFuses(f: word);
  function AVRreadExtFuses(): byte;
  procedure AVRprogExtFuses(b: byte);
  function AVRreadLockBits(): byte;
  procedure AVRprogLockBits(b: byte);

  procedure AVRblInit();
  procedure AVRblStart();
  procedure AVRblWritePage(d: array of word; pa: word; ps: byte);

implementation

procedure Delay();
var i: integer;
begin
//  if cbSleep.Checked then
//  begin
//    Sleep(1);
//  end else
  begin
    for i := 1 to 10000 * delaycnt do;
  end;
end;

procedure SCK(t: boolean); forward;

// -----------------------------------------------------------------------------
// --- LPT port routines -------------------------------------------------------
// -----------------------------------------------------------------------------

const LptRSTmask = $20;         // hardware pin 7
const LptSCKmask = $40;         // hardware pin 8
const LptMOSImask = $80;        // hardware pin 9
const LptMISOmask = $40;       // hardware pin 10

procedure LptOutA (b: byte);
begin
  asm
    mov dx, lptp
    mov al, [b]
    out dx, al
  end;
end;

function LptInA: byte;
begin
  asm
    mov dx, lptp
    in  al, dx
    mov Result, al
  end;
end;

function LptInC: byte;
begin
  asm
    mov dx, lptp+1
    in  al, dx
    xor al, 80h
    mov Result, al
  end;
end;

procedure LptRST(t: boolean);
begin
  if t then LptOutA(LptInA or LptRSTmask)
       else LptOutA(LptInA and (not LptRSTmask));
end;

procedure LptSCK(t: boolean);
begin
  if t then LptOutA(LptInA or LptSCKmask)
       else LptOutA(LptInA and (not LptSCKmask));
end;

procedure LptMOSI(t: boolean);
begin
  if t then LptOutA(LptInA or LptMOSImask)
       else LptOutA(LptInA and (not LptMOSImask));
end;

function LptMISO():boolean;
begin
  Result := (LptInC() and LptMISOmask) > 0;
end;

procedure OpenLpt();
var hx: THandle;
begin
  hx := CreateFile('\\.\\giveio', GENERIC_READ, 0, nil, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
  if hx = INVALID_HANDLE_VALUE then raise Exception.Create('GiveIO driver not installed');
  CloseHandle(hx);

//  LptOutA($ff);
  SCK(false);
  LptRST(true);         // RESET goes high
  Delay();
  LptRST(false);        // RESET goes back low (active state)
end;

procedure CloseLpt();
begin
// nothing to do for now
end;

// -----------------------------------------------------------------------------
// --- COM port routines -------------------------------------------------------
// -----------------------------------------------------------------------------

var h: THandle;

procedure ComSCK(t: boolean);
begin
  if t then EscapeCommFunction(h, SETRTS)
       else EscapeCommFunction(h, CLRRTS);
end;

procedure ComMOSI(t: boolean);
begin
  if t then EscapeCommFunction(h, SETDTR)
       else EscapeCommFunction(h, CLRDTR);
end;

function ComMISO(): boolean;
var c: cardinal;
begin
  GetCommModemStatus (h, c);
  Result := (c and MS_CTS_ON) > 0;
end;

procedure OpenCom(c: char);
begin
  CloseCom();

  h := CreateFile(PChar('COM' + c), GENERIC_READ OR GENERIC_WRITE, 0, nil, OPEN_EXISTING, 0, THandle (nil));
  if h = INVALID_HANDLE_VALUE then raise Exception.Create('Could not open COM' + c);

  SCK(false);
//  EscapeCommFunction(h, SETBREAK); // set TX line high
end;

procedure CloseCom();
begin
  if h <> INVALID_HANDLE_VALUE then CloseHandle(h);
  h := INVALID_HANDLE_VALUE;
end;

// -----------------------------------------------------------------------------
// --- common control routines -------------------------------------------------
// -----------------------------------------------------------------------------

procedure SCK(t: boolean);
begin
  if invsck then t := not t;

  if h = INVALID_HANDLE_VALUE
  then LptSCK(t)
  else ComSCK(t);
end;

procedure MOSI(t: boolean);
begin
  if invmosi then t := not t;

  if h = INVALID_HANDLE_VALUE
  then LptMOSI(t)
  else ComMOSI(t);
end;

function MISO(): boolean;
begin
  if h = INVALID_HANDLE_VALUE
  then Result := LptMISO()
  else Result := ComMISO();

  if invmiso then Result := not Result;
end;

procedure txByte(b: byte);
var i: integer;
begin
  for i := 0 to 7 do
  begin
    MOSI((b and ($80 shr i)) > 0);
    SCK(true);
    Delay();
    SCK(false);
    Delay();
  end;
end;

function rxByte(): byte;
var i: integer;
    b: byte;
begin
  b := 0;

  for i := 0 to 7 do
  begin
    if MISO then b := b + 1;
    if i < 7 then b := b shl 1;

    SCK(true);
    Delay();
    SCK(false);
    Delay();
  end;

  Result := b;
end;

// -----------------------------------------------------------------------------
// --- AVR routines ------------------------------------------------------------
// -----------------------------------------------------------------------------

function AVRdeviceName(a: integer): string;
var i: integer;
begin
  Result := 'unknown device';

  for i := 0 to devnum-1 do
  begin
    if devsig[i] = a then
    begin
      Result := devnames[i];
      Exit;
    end;
  end;
end;

procedure AVRprogEnable();
begin
  txByte($AC);
  txByte($53);
  txByte(0);
  txByte(0);
end;

function AVRreadSignature(a: byte): byte;
begin
  txByte($30);
  txByte(0);
  txByte(a);
  Result := rxByte();
end;


procedure AVRchipErase();
begin
  txByte($AC);
  txByte($80);
  txByte(0);
  txByte(0);

  Sleep(100);
end;

function AVRreadEE(a: integer): byte;
var ah,al: byte;
begin
  ah := (a shr 8) and $03;
  al := a and $ff;

  txByte($A0);
  txByte(ah);
  txByte(al);
  Result := rxByte();
end;

procedure AVRwriteEE(a: integer; b: byte);
var i: integer;
    ah,al: byte;
begin
  if (b = $ff)or(b = AVRreadEE(a)) then Exit;
  ah := (a shr 8) and $03;
  al := a and $ff;

  txByte($C0);
  txByte(ah);
  txByte(al);
  txByte(b);

  for i := 1 to 10 do
  begin
    Sleep(1);
    if AVRreadEE(a) = b then break;
  end;
  if i = 11 then raise Exception.Create('Could not write EEPROM to location 0x' + inttohex(a,3));
end;

function AVRreadProgByte(a: integer; high: boolean): byte;
var al,ah,hl: byte;
begin
  ah := (a shr 8) and $ff;
  al := a and $ff;

  if high then hl := $28
          else hl := $20;

  txByte(hl);
  txByte(ah);
  txByte(al);
  Result := rxByte();
end;

function AVRreadProg(a: integer): word;
begin
  Result := AVRreadProgByte(a, true) * $100 + AVRreadProgByte(a, false);
end;

procedure AVRwriteProg(a: integer; w: word);
var al,ah,dl,dh: byte;
    i: integer;
begin
  ah := (a shr 8) and $3f;
  al := a and $ff;
  dh := w shr 8;
  dl := w and $ff;

  if (dh <> $ff)and(dh <> AVRreadProgByte(a, true)) then
  begin
    txByte($48);  // write high byte
    txByte(ah);
    txByte(al);
    txByte(dh);

    // Sleep(10);
    for i := 1 to 10 do
    begin
      Sleep(1);
      if AVRreadProgByte(a, true) = dh then break;
    end;
    if i = 11 then raise Exception.Create('Could not write Flash high byte to location 0x' + inttohex(a,4));
  end;

  if (dl <> $ff)and(dl <> AVRreadProgByte(a, false)) then
  begin
    txByte($40);
    txByte(ah);
    txByte(al);   // write low byte
    txByte(dl);

    // Sleep(10);
    for i := 1 to 10 do
    begin
      Sleep(1);
      if AVRreadProgByte(a, false) = dl then break;
    end;
    if i = 11 then raise Exception.Create('Could not write Flash low byte to location 0x' + inttohex(a,4));
  end;
end;

procedure AVRlock();
begin
  txByte($AC);          // program lock bits command
  txByte($F9);          // lock both LB1 and LB2
  txByte(0);
  txByte(0);
end;

function PageEmpty(var d: array of word; ps: byte): boolean;
var i: integer;
begin
  Result := true;

  for i := 0 to ps-1 do
  begin
    if d[i] <> $ffff then
    begin
      Result := false;
      Break;
    end;
  end;
end;

procedure AVRwritePage(d: array of word; pa: word; ps: byte);
var a: byte;
begin
  if PageEmpty(d, ps) then Exit;

  for a := 0 to ps-1 do
  begin
    txByte($40);    // write low byte before high (requirement) !
    txByte(0);
    txByte(a);
    txByte(Lo(d[a]));

    txByte($48);    // write high byte to the same address
    txByte(0);
    txByte(a);
    txByte(Hi(d[a]));
  end;

  while ps > 1 do       // shift page address left by log2(page size)
  begin
    pa := pa shl 1;
    ps := ps shr 1;
  end;

  txByte($4c);            // write program memory page
  txByte(Hi(pa));             // high 4 bits of page addr on the right
  txByte(Lo(pa));             // low 3 bits of page addr on the left
  txByte(0);

  Sleep(15);               // supposed to wait 4.5ms
end;

function AVRreadFuses(): word;
var fhb,flb: byte;
begin
  // read fuse high byte
  txByte($58);
  txByte($08);
  txByte(0);
  fhb := rxByte();

  // read fuse low byte
  txByte($50);
  txByte(0);
  txByte(0);
  flb := rxByte();

  Result := fhb * $100 + flb;
end;

procedure AVRprogFuses(f: word);
begin
  // write fuse high byte
  txByte($AC);
  txByte($A8);
  txByte(0);
  txByte(Hi(f));

  Sleep(10);

  // write fuse low byte
  txByte($AC);
  txByte($A0);
  txByte(0);
  txByte(Lo(f));

  Sleep(100);
end;

function AVRreadExtFuses(): byte;
begin
  txByte($50);
  txByte($08);
  txByte(0);
  Result := rxByte();
end;

procedure AVRprogExtFuses(b: byte);
begin
  txByte($AC);
  txByte($A4);
  txByte(0);
  txByte(b);
end;

function AVRreadLockBits(): byte;
begin
  txByte($58);
  txByte(0);
  txByte(0);
  Result := rxByte() and $3f;
end;

procedure AVRprogLockBits(b: byte);
begin
  txByte($AC);
  txByte($E0);
  txByte(0);
  txByte(b or $C0);
end;

// -----------------------------------------------------------------------------
// --- Bootloader routines -----------------------------------------------------
// -----------------------------------------------------------------------------

procedure XmodemCrc(data: byte; var crc: word);
var i: integer;
begin
  crc := crc xor (word(data) shl 8);
  for i := 0 to 7 do
  begin
    if (crc and $8000)>0 then crc := (crc shl 1) xor $1021
                         else crc := crc shl 1;
  end;
end;

procedure AVRblInit;
var dcb: TDCB;
    timeouts: tcommtimeouts;
    abp: cardinal;
    s: string;
begin
  if h = INVALID_HANDLE_VALUE then
    raise Exception.Create('COM port not opened');

//  EscapeCommFunction(h, CLRBREAK);  // normalize TX line

  if not GetCommState(h, DCB) then
    raise Exception.Create('Error: GetCommState');

  DCB.BaudRate := 115200;
  DCB.Parity   := NOPARITY;
  DCB.ByteSize := 8;
  DCB.StopBits := ONESTOPBIT;

  if not SetCommState(h, DCB) then
    raise Exception.Create('Error: SetCommState');

  if not SetupComm(h, 4096, 4096) then
    raise Exception.Create('Error: SetupComm'); // Raise an exception

  if not GetCommTimeouts(h, Timeouts) then
    raise Exception.Create('Error: GetCommTimeouts');

  Timeouts.ReadIntervalTimeout := 100;  // ce vec kot 500 ms pretece med dvema znakoma, timeout
  Timeouts.ReadTotalTimeoutMultiplier := 10;     // koliko ms naj doda za vsak prejet znak
  Timeouts.ReadTotalTimeoutConstant := 1000;       // koliko najmanj ms naj caka, naj ne bo 0 !!
  Timeouts.WriteTotalTimeoutMultiplier := 10;
  Timeouts.WriteTotalTimeoutConstant := 1000;

  if not SetCommTimeouts(h, Timeouts) then
    raise Exception.Create('Error: SetCommTimeouts');

  if not SetCommMask(h, EV_TXEMPTY) then
    raise Exception.Create('Error: SetCommMask');

//  if not PurgeComm(h, PURGE_TXCLEAR or PURGE_RXCLEAR) then
//    raise Exception.Create('Error: PurgeComm');

  setlength(s, 100);
  repeat ReadFile(h, PChar(s)^, 100, abp, nil) until abp < 100;
end;

procedure AVRblStart();
var abp: cardinal;
    s: string;
begin
  setlength(s, 50);
  ReadFile(h, PChar(s)^, length(s), abp, nil);
  setlength(s, abp);
  if pos('1.0', s) = 0 then Exit;

  s := 'JeTaM';
  WriteFile(h, PChar(s)^, length(s), abp, nil);         // send magic word

  setlength(s, 50);
  ReadFile(h, PChar(s)^, length(s), abp, nil);          // wait for RDY
  setlength(s, abp);
  if (abp = 0) or (Pos('RDY',s) = 0) then raise Exception.Create('No RDY');
end;

procedure AVRblWritePage(d: array of word; pa: word; ps: byte);
var s: string;
    abp: Cardinal;
    i,j: integer;
    w: word;
begin
  if PageEmpty(d, ps) then Exit;

  if h = INVALID_HANDLE_VALUE then
    raise Exception.Create('COM port not opened');

  s := 'W';
  WriteFile(h, PChar(s)^, length(s), abp, nil);

  setlength(s, 50);
  ReadFile(h, PChar(s)^, length(s), abp, nil);
  setlength(s, abp);
  if (abp = 0) or (Pos('OK',s) = 0) then raise Exception.Create('No OK after W');

  WriteFile(h, d, 2*ps, abp, nil);

  setlength(s, 50);
  ReadFile(h, PChar(s)^, length(s), abp, nil);
  setlength(s, abp);
  if (abp = 0) or (Pos('OK',s) = 0) then raise Exception.Create('No OK after page data');

  // crc / flash page number verification
  w := 0;
  for i := 0 to ps-1 do
  begin
    xmodemcrc(d[i], w);
    xmodemcrc(d[i] shr 8, w);
  end;

  i := Pos(IntToHex(pa, 2),s);
  j := Pos(IntToHex(w, 4), s);
  if (i = 0) or (j = 0) or (i >= j) then raise Exception.Create('Flash CRC/page num error');
end;

// -----------------------------------------------------------------------------
// --- Unit initialization -----------------------------------------------------
// -----------------------------------------------------------------------------

begin
  h := INVALID_HANDLE_VALUE;
  invsck := false;
  invmiso := false;
  invmosi := false;
  delaycnt := 1;
end.
