void flash_write_page(uint16_t g, char* buf)
{
	uint16_t i;
	uint32_t page = g * SPM_PAGESIZE;
	
	boot_page_erase(page);	// erase page
	boot_spm_busy_wait();	// wait while operation is in progress
	
	uint16_t page_blank = 0xffff;
    for (i=0; i<SPM_PAGESIZE; i+=2)	// load temp page buffer
    {
        uint16_t w = (*buf++);
        w += (*buf++) << 8;
		page_blank &= w;
        boot_page_fill (page + i, w);
    }	
	
	if (page_blank != 0xffff) {	// write page if not empty
		boot_page_write (page);     // Store buffer in flash page.
		boot_spm_busy_wait();       // Wait until the memory is written.
	}

    // Reenable RWW-section again. We need this if we want to jump back
    // to the application after bootloading.

    boot_rww_enable ();
}

uint16_t flash_page_crc(uint16_t g)
{
	uint32_t page = g * SPM_PAGESIZE;
	uint16_t crc = 0;
	uint16_t i;
	
	for (i = 0; i < SPM_PAGESIZE; i++) {
		crc = _crc_xmodem_update(crc, pgm_read_byte(page++));
	}
	
	return crc;
}
