
void serial_init(void)
{
#ifdef _AVR_IOM128_H_
	UBRR0H = 0;
	UBRR0L = 5;		// 5 = 115k2 @ 11.0592 MHz
	UCSR0A = 0;		// normal UART speed, no multi-proc comm mode
	UCSR0B = 0x98;	// enable RX/TX, enable RX int
	UCSR0C = 0x06;	// 8 bit characters
#else
	UBRRH = 0;
	UBRRL = 5;		// 5 = 115k2 @ 11.0592 MHz
	UCSRA = 0;		// normal UART speed, no multi-proc comm mode
	UCSRB = 0x98;	// enable RX/TX, enable RX int
	UCSRC = 0x86;	// 8 bit characters
#endif
}

void serial_disable(void)
{
#ifdef _AVR_IOM128_H_
	UBRR0H = 0;
	UBRR0L = 0;
	UCSR0A = 0;
	UCSR0B = 0;
	UCSR0C = 0x06;
#else
	UBRRH = 0;
	UBRRL = 0;
	UCSRA = 0;
	UCSRB = 0;
	UCSRC = 0x86;
#endif
}

void comsendchar(const char a)
{
#ifdef _AVR_IOM128_H_
	loop_until_bit_is_set(UCSR0A, UDRE0);
	UDR0 = a;
#else
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = a;
#endif
}

#ifdef _AVR_IOM128_H_
void comsendstr_P(uint32_t s)
{
	uint8_t c;
	while ((c = pgm_read_byte_far(s++)))	{
		comsendchar(c);
	}
}
#else
void comsendstr_P(PGM_P s)
{
	uint8_t c;
	while ((c = pgm_read_byte(s++)))	{
		comsendchar(c);
	}
}
#endif

void comsendstr(const char* s)
{
	while (*s)	{
		comsendchar(*s);
		s++;
	}
}

void comsendnstrx(char* s, int n)
{
	while (n > 0) {
		comsendchar(*s);
		s++;
		n--;
	}
}

void comsendnib(const uint8_t b)
{
	uint8_t n = b & 0x0f;
	if (n < 10) {
		comsendchar('0' + n);
	} else {
		comsendchar('A' + n - 10);
	}
}

void comsendhex(const uint8_t n)
{
	comsendnib(n >> 4);
	comsendnib(n);
}

void comsendhex16(const uint16_t n)
{
	comsendhex(n >> 8);
	comsendhex(n);
}


#ifdef _AVR_IOM128_H_
SIGNAL(SIG_UART0_RECV)
{
	serial_rx_int(UDR0);
}
#else
SIGNAL(SIG_UART_RECV)
{
	serial_rx_int(UDR);
}
#endif

